@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -24,5 +23,6 @@
  */
 
 $string['calendar_upcoming:addinstance'] = 'Add a new upcoming events block';
-$string['calendar_upcoming:myaddinstance'] = 'Add a new upcoming events block to the My Moodle page';
+$string['calendar_upcoming:myaddinstance'] = 'Add a new upcoming events block to Dashboard';
+$string['gotocalendar'] = 'Go to calendar...';
 $string['pluginname'] = 'Upcoming events';
