@@ -316,7 +316,8 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
                 }
             } else if (isset($expectations['exception'])) {
                 $exception = $expectations['exception'];
-                $this->setExpectedException($exception['type'], $exception['message']);
+                $this->expectException($exception['type']);
+                $this->expectExceptionMessage($exception['message']);
             } else {
                 // Failed, only canview and exception are supported.
                 $this->markTestIncomplete('Incomplete, only canview and exception are supported');
@@ -362,12 +363,17 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
 
         $this->resetAfterTest(true);
 
+        $timenow = time();
         $coursedata1 = array(
+            'fullname'         => '<b>Course 1</b>',                // Adding tags here to check that external_format_string works.
+            'shortname'         => '<b>Course 1</b>',               // Adding tags here to check that external_format_string works.
             'summary'          => 'Lightwork Course 1 description',
             'summaryformat'    => FORMAT_MOODLE,
             'lang'             => 'en',
             'enablecompletion' => true,
-            'showgrades'       => true
+            'showgrades'       => true,
+            'startdate'        => $timenow,
+            'enddate'          => $timenow + WEEKSECS
         );
 
         $course1 = self::getDataGenerator()->create_course($coursedata1);
@@ -401,6 +407,8 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
 
         // Check there are no differences between $course1 properties and course values returned by the webservice
         // only for those fields listed in the $coursedata1 array.
+        $course1->fullname = external_format_string($course1->fullname, $contexts[$course1->id]->id);
+        $course1->shortname = external_format_string($course1->shortname, $contexts[$course1->id]->id);
         foreach ($enrolledincourses as $courseenrol) {
             if ($courseenrol['id'] == $course1->id) {
                 foreach ($coursedata1 as $fieldname => $value) {
@@ -428,7 +436,9 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
         $this->assertNotEmpty($studentrole);
 
         $course1 = self::getDataGenerator()->create_course();
-        $course2 = self::getDataGenerator()->create_course();
+        $coursedata = new stdClass();
+        $coursedata->visible = 0;
+        $course2 = self::getDataGenerator()->create_course($coursedata);
 
         // Add enrolment methods for course.
         $instanceid1 = $selfplugin->add_instance($course1, array('status' => ENROL_INSTANCE_ENABLED,
@@ -445,6 +455,8 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
         $enrolmentmethods = $DB->get_records('enrol', array('courseid' => $course1->id, 'status' => ENROL_INSTANCE_ENABLED));
         $this->assertCount(2, $enrolmentmethods);
 
+        $this->setAdminUser();
+
         // Check if information is returned.
         $enrolmentmethods = core_enrol_external::get_course_enrolment_methods($course1->id);
         $enrolmentmethods = external_api::clean_returnvalue(core_enrol_external::get_course_enrolment_methods_returns(),
@@ -474,6 +486,15 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals('self', $enrolmentmethod['type']);
         $this->assertTrue($enrolmentmethod['status']);
         $this->assertEquals('enrol_self_get_instance_info', $enrolmentmethod['wsfunction']);
+
+        // Try to retrieve information using a normal user for a hidden course.
+        $user = self::getDataGenerator()->create_user();
+        $this->setUser($user);
+        try {
+            core_enrol_external::get_course_enrolment_methods($course2->id);
+        } catch (moodle_exception $e) {
+            $this->assertEquals('coursehidden', $e->errorcode);
+        }
     }
 
     public function get_enrolled_users_setup($capability) {
@@ -548,6 +569,8 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
     /**
      * Test get_enrolled_users from core_enrol_external with capability to
      * viewparticipants removed.
+     *
+     * @expectedException moodle_exception
      */
     public function test_get_enrolled_users_without_capability() {
         $capability = 'moodle/course:viewparticipants';
@@ -555,7 +578,6 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
 
         // Call without required capability.
         $this->unassignUserCapability($capability, $data->context->id, $data->roleid);
-        $this->setExpectedException('moodle_exception');
         $categories = core_enrol_external::get_enrolled_users($data->course->id);
     }
 
@@ -660,4 +682,151 @@ class core_enrol_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals($data->student1->id, $expecteduser['id']);
     }
 
+    /**
+     * Test for core_enrol_external::edit_user_enrolment().
+     */
+    public function test_edit_user_enrolment() {
+        global $DB;
+
+        $this->resetAfterTest(true);
+        $datagen = $this->getDataGenerator();
+
+        /** @var enrol_manual_plugin $manualplugin */
+        $manualplugin = enrol_get_plugin('manual');
+        $this->assertNotNull($manualplugin);
+
+        $studentroleid = $DB->get_field('role', 'id', ['shortname' => 'student'], MUST_EXIST);
+        $teacherroleid = $DB->get_field('role', 'id', ['shortname' => 'editingteacher'], MUST_EXIST);
+        $course = $datagen->create_course();
+        $user = $datagen->create_user();
+        $teacher = $datagen->create_user();
+
+        $instanceid = null;
+        $instances = enrol_get_instances($course->id, true);
+        foreach ($instances as $inst) {
+            if ($inst->enrol == 'manual') {
+                $instanceid = (int)$inst->id;
+                break;
+            }
+        }
+        if (empty($instanceid)) {
+            $instanceid = $manualplugin->add_default_instance($course);
+            if (empty($instanceid)) {
+                $instanceid = $manualplugin->add_instance($course);
+            }
+        }
+        $this->assertNotNull($instanceid);
+
+        $instance = $DB->get_record('enrol', ['id' => $instanceid], '*', MUST_EXIST);
+        $manualplugin->enrol_user($instance, $user->id, $studentroleid, 0, 0, ENROL_USER_ACTIVE);
+        $manualplugin->enrol_user($instance, $teacher->id, $teacherroleid, 0, 0, ENROL_USER_ACTIVE);
+        $ueid = (int)$DB->get_field(
+            'user_enrolments',
+            'id',
+            ['enrolid' => $instance->id, 'userid' => $user->id],
+            MUST_EXIST
+        );
+
+        // Login as teacher.
+        $this->setUser($teacher);
+
+        $now = new DateTime();
+        $nowtime = $now->getTimestamp();
+
+        // Invalid data.
+        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_ACTIVE, $nowtime, $nowtime);
+        $data = external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
+        $this->assertFalse($data['result']);
+        $this->assertNotEmpty($data['errors']);
+
+        // Valid data.
+        $nextmonth = clone($now);
+        $nextmonth->add(new DateInterval('P1M'));
+        $nextmonthtime = $nextmonth->getTimestamp();
+        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_ACTIVE, $nowtime, $nextmonthtime);
+        $data = external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
+        $this->assertTrue($data['result']);
+        $this->assertEmpty($data['errors']);
+
+        // Check updated user enrolment.
+        $ue = $DB->get_record('user_enrolments', ['id' => $ueid], '*', MUST_EXIST);
+        $this->assertEquals(ENROL_USER_ACTIVE, $ue->status);
+        $this->assertEquals($nowtime, $ue->timestart);
+        $this->assertEquals($nextmonthtime, $ue->timeend);
+
+        // Suspend user.
+        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_SUSPENDED);
+        $data = external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
+        $this->assertTrue($data['result']);
+        $this->assertEmpty($data['errors']);
+
+        // Check updated user enrolment.
+        $ue = $DB->get_record('user_enrolments', ['id' => $ueid], '*', MUST_EXIST);
+        $this->assertEquals(ENROL_USER_SUSPENDED, $ue->status);
+    }
+
+    /**
+     * Test for core_enrol_external::unenrol_user_enrolment().
+     */
+    public function test_unenerol_user_enrolment() {
+        global $DB;
+
+        $this->resetAfterTest(true);
+        $datagen = $this->getDataGenerator();
+
+        /** @var enrol_manual_plugin $manualplugin */
+        $manualplugin = enrol_get_plugin('manual');
+        $this->assertNotNull($manualplugin);
+
+        $studentroleid = $DB->get_field('role', 'id', ['shortname' => 'student'], MUST_EXIST);
+        $teacherroleid = $DB->get_field('role', 'id', ['shortname' => 'editingteacher'], MUST_EXIST);
+        $course = $datagen->create_course();
+        $user = $datagen->create_user();
+        $teacher = $datagen->create_user();
+
+        $instanceid = null;
+        $instances = enrol_get_instances($course->id, true);
+        foreach ($instances as $inst) {
+            if ($inst->enrol == 'manual') {
+                $instanceid = (int)$inst->id;
+                break;
+            }
+        }
+        if (empty($instanceid)) {
+            $instanceid = $manualplugin->add_default_instance($course);
+            if (empty($instanceid)) {
+                $instanceid = $manualplugin->add_instance($course);
+            }
+        }
+        $this->assertNotNull($instanceid);
+
+        $instance = $DB->get_record('enrol', ['id' => $instanceid], '*', MUST_EXIST);
+        $manualplugin->enrol_user($instance, $user->id, $studentroleid, 0, 0, ENROL_USER_ACTIVE);
+        $manualplugin->enrol_user($instance, $teacher->id, $teacherroleid, 0, 0, ENROL_USER_ACTIVE);
+        $ueid = (int)$DB->get_field(
+            'user_enrolments',
+            'id',
+            ['enrolid' => $instance->id, 'userid' => $user->id],
+            MUST_EXIST
+        );
+
+        // Login as teacher.
+        $this->setUser($teacher);
+
+        // Invalid data by passing invalid ueid.
+        $data = core_enrol_external::unenrol_user_enrolment(101010);
+        $data = external_api::clean_returnvalue(core_enrol_external::unenrol_user_enrolment_returns(), $data);
+        $this->assertFalse($data['result']);
+        $this->assertNotEmpty($data['errors']);
+
+        // Valid data.
+        $data = core_enrol_external::unenrol_user_enrolment($ueid);
+        $data = external_api::clean_returnvalue(core_enrol_external::unenrol_user_enrolment_returns(), $data);
+        $this->assertTrue($data['result']);
+        $this->assertEmpty($data['errors']);
+
+        // Check unenrol user enrolment.
+        $ue = $DB->count_records('user_enrolments', ['id' => $ueid]);
+        $this->assertEquals(0, $ue);
+    }
 }
