@@ -33,10 +33,12 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         ENTER: 13,
         SPACE: 32,
         ESCAPE: 27,
-        COMMA: 188,
+        COMMA: 44,
         UP: 38
     };
 
+    var uniqueId = $.now();
+
     /**
      * Make an item in the selection list "active".
      *
@@ -450,10 +452,18 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         updateSelectionList(options, state, originalSelect);
         // Notifiy that the selection changed.
         notifyChange(originalSelect);
-        // Clear the input element.
-        inputElement.val('');
-        // Close the list of suggestions.
-        closeSuggestions(state);
+
+        if (options.closeSuggestionsOnSelect) {
+            // Clear the input element.
+            inputElement.val('');
+            // Close the list of suggestions.
+            closeSuggestions(state);
+        } else {
+            // Focus on the input element so the suggestions does not auto-close.
+            inputElement.focus();
+            // Remove the last selected item from the suggestions list.
+            updateSuggestions(options, state, inputElement.val(), originalSelect);
+        }
     };
 
     /**
@@ -536,21 +546,13 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
                                 updateAjax(e, options, state, originalSelect, ajaxHandler);
                             });
                         } else {
-                            // Else - open the suggestions list.
+                            // Open the suggestions list.
                             updateSuggestions(options, state, inputElement.val(), originalSelect);
                         }
                     }
                     // We handled this event, so prevent it.
                     e.preventDefault();
                     return false;
-                case KEYS.COMMA:
-                    if (options.tags) {
-                        // If we are allowing tags, comma should create a tag (or enter).
-                        createItem(options, state, originalSelect);
-                    }
-                    // We handled this event, so prevent it.
-                    e.preventDefault();
-                    return false;
                 case KEYS.UP:
                     // Choose the previous active item.
                     activatePreviousItem(state);
@@ -581,6 +583,19 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
             }
             return true;
         });
+        // Support multi lingual COMMA keycode (44).
+        inputElement.on('keypress', function(e) {
+            if (e.keyCode === KEYS.COMMA) {
+                if (options.tags) {
+                    // If we are allowing tags, comma should create a tag (or enter).
+                    createItem(options, state, originalSelect);
+                }
+                // We handled this event, so prevent it.
+                e.preventDefault();
+                return false;
+            }
+            return true;
+        });
         // Handler used to force set the value from behat.
         inputElement.on('behat:set-value', function() {
             var suggestionsElement = $(document.getElementById(state.suggestionsId));
@@ -608,11 +623,18 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         });
         if (options.showSuggestions) {
             var arrowElement = $(document.getElementById(state.downArrowId));
-            arrowElement.on('click', function() {
+            arrowElement.on('click', function(e) {
                 // Prevent the close timer, or we will open, then close the suggestions.
                 inputElement.focus();
-                // Show the suggestions list.
-                updateSuggestions(options, state, inputElement.val(), originalSelect);
+                // Handle ajax population of suggestions.
+                if (!inputElement.val() && options.ajax) {
+                    require([options.ajax], function(ajaxHandler) {
+                        updateAjax(e, options, state, originalSelect, ajaxHandler);
+                    });
+                } else {
+                    // Else - open the suggestions list.
+                    updateSuggestions(options, state, inputElement.val(), originalSelect);
+                }
             });
         }
 
@@ -667,19 +689,40 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         });
         // Whenever the input field changes, update the suggestion list.
         if (options.showSuggestions) {
-            inputElement.on('input', function(e) {
-                var query = $(e.currentTarget).val();
-                var last = $(e.currentTarget).data('last-value');
-                // IE11 fires many more input events than required - even when the value has not changed.
-                // We need to only do this for real value changed events or the suggestions will be
-                // unclickable on IE11 (because they will be rebuilt before the click event fires).
-                // Note - because of this we cannot close the list when the query is empty or it will break
-                // on IE11.
-                if (last !== query) {
-                    updateSuggestions(options, state, query, originalSelect);
-                }
-                $(e.currentTarget).data('last-value', query);
-            });
+            // If this field uses ajax, set it up.
+            if (options.ajax) {
+                require([options.ajax], function(ajaxHandler) {
+                    var throttleTimeout = null;
+                    var handler = function(e) {
+                        updateAjax(e, options, state, originalSelect, ajaxHandler);
+                    };
+
+                    // For input events, we do not want to trigger many, many updates.
+                    var throttledHandler = function(e) {
+                        if (throttleTimeout !== null) {
+                            window.clearTimeout(throttleTimeout);
+                            throttleTimeout = null;
+                        }
+                        throttleTimeout = window.setTimeout(handler.bind(this, e), 300);
+                    };
+                    // Trigger an ajax update after the text field value changes.
+                    inputElement.on("input", throttledHandler);
+                });
+            } else {
+                inputElement.on('input', function(e) {
+                    var query = $(e.currentTarget).val();
+                    var last = $(e.currentTarget).data('last-value');
+                    // IE11 fires many more input events than required - even when the value has not changed.
+                    // We need to only do this for real value changed events or the suggestions will be
+                    // unclickable on IE11 (because they will be rebuilt before the click event fires).
+                    // Note - because of this we cannot close the list when the query is empty or it will break
+                    // on IE11.
+                    if (last !== query) {
+                        updateSuggestions(options, state, query, originalSelect);
+                    }
+                    $(e.currentTarget).data('last-value', query);
+                });
+            }
         }
     };
 
@@ -698,8 +741,11 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
          * @param {Boolean} caseSensitive - If search has to be made case sensitive.
          * @param {Boolean} showSuggestions - If suggestions should be shown
          * @param {String} noSelectionString - Text to display when there is no selection
+         * @param {Boolean} closeSuggestionsOnSelect - Whether to close the suggestions immediately after making a selection.
+         * @return {Promise}
          */
-        enhance: function(selector, tags, ajax, placeholder, caseSensitive, showSuggestions, noSelectionString) {
+        enhance: function(selector, tags, ajax, placeholder, caseSensitive, showSuggestions, noSelectionString,
+                          closeSuggestionsOnSelect) {
             // Set some default values.
             var options = {
                 selector: selector,
@@ -732,22 +778,34 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
             var originalSelect = $(selector);
             if (!originalSelect) {
                 log.debug('Selector not found: ' + selector);
-                return;
+                return false;
             }
 
+            originalSelect.css('visibility', 'hidden').attr('aria-hidden', true);
+
             // Hide the original select.
-            originalSelect.hide().attr('aria-hidden', true);
 
             // Find or generate some ids.
             var state = {
                 selectId: originalSelect.attr('id'),
-                inputId: 'form_autocomplete_input-' + $.now(),
-                suggestionsId: 'form_autocomplete_suggestions-' + $.now(),
-                selectionId: 'form_autocomplete_selection-' + $.now(),
-                downArrowId: 'form_autocomplete_downarrow-' + $.now()
+                inputId: 'form_autocomplete_input-' + uniqueId,
+                suggestionsId: 'form_autocomplete_suggestions-' + uniqueId,
+                selectionId: 'form_autocomplete_selection-' + uniqueId,
+                downArrowId: 'form_autocomplete_downarrow-' + uniqueId
             };
+
+            // Increment the unique counter so we don't get duplicates ever.
+            uniqueId++;
+
             options.multiple = originalSelect.attr('multiple');
 
+            if (typeof closeSuggestionsOnSelect !== "undefined") {
+                options.closeSuggestionsOnSelect = closeSuggestionsOnSelect;
+            } else {
+                // If not specified, this will close suggestions by default for single-select elements only.
+                options.closeSuggestionsOnSelect = !options.multiple;
+            }
+
             var originalLabel = $('[for=' + state.selectId + ']');
             // Create the new markup and insert it after the select.
             var suggestions = [];
@@ -764,45 +822,24 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
             var renderDatalist = templates.render('core/form_autocomplete_suggestions', context);
             var renderSelection = templates.render('core/form_autocomplete_selection', context);
 
-            $.when(renderInput, renderDatalist, renderSelection).done(function(input, suggestions, selection) {
-                // Add our new UI elements to the page.
+            return $.when(renderInput, renderDatalist, renderSelection).then(function(input, suggestions, selection) {
+                originalSelect.hide();
                 originalSelect.after(suggestions);
                 originalSelect.after(input);
                 originalSelect.after(selection);
+
                 // Update the form label to point to the text input.
                 originalLabel.attr('for', state.inputId);
                 // Add the event handlers.
                 addNavigation(options, state, originalSelect);
 
-                var inputElement = $(document.getElementById(state.inputId));
                 var suggestionsElement = $(document.getElementById(state.suggestionsId));
                 // Hide the suggestions by default.
                 suggestionsElement.hide().attr('aria-hidden', true);
 
-                // If this field uses ajax, set it up.
-                if (options.ajax) {
-                    require([options.ajax], function(ajaxHandler) {
-                        var throttleTimeout = null;
-                        var handler = function(e) {
-                            updateAjax(e, options, state, originalSelect, ajaxHandler);
-                        };
-
-                        // For input events, we do not want to trigger many, many updates.
-                        var throttledHandler = function(e) {
-                            if (throttleTimeout !== null) {
-                                window.clearTimeout(throttleTimeout);
-                                throttleTimeout = null;
-                            }
-                            throttleTimeout = window.setTimeout(handler.bind(this, e), 300);
-                        };
-                        // Trigger an ajax update after the text field value changes.
-                        inputElement.on("input", throttledHandler);
-                        var arrowElement = $(document.getElementById(state.downArrowId));
-                        arrowElement.on("click", handler);
-                    });
-                }
                 // Show the current values in the selection list.
                 updateSelectionList(options, state, originalSelect);
+                return true;
             });
         }
     };
