@@ -257,6 +257,15 @@ $definitions = array(
         'simpledata' => true,
     ),
 
+    // Cache system-wide role definitions.
+    'roledefs' => array(
+        'mode' => cache_store::MODE_APPLICATION,
+        'simplekeys' => true,
+        'simpledata' => true,
+        'staticacceleration' => true,
+        'staticaccelerationsize' => 30,
+    ),
+
     // Caches plugins existing functions by function name and file.
     // Set static acceleration size to 5 to load a few functions.
     'plugin_functions' => array(
@@ -303,6 +312,15 @@ $definitions = array(
         ),
     ),
 
+    // Caches contexts with insights.
+    'contextwithinsights' => array(
+        'mode' => cache_store::MODE_APPLICATION,
+        'simplekeys' => true,
+        'simpledata' => true,
+        'staticacceleration' => true,
+        'staticaccelerationsize' => 1
+    ),
+
     // Caches message processors.
     'message_processors_enabled' => array(
         'mode' => cache_store::MODE_APPLICATION,
@@ -320,6 +338,15 @@ $definitions = array(
         'datasource' => '\core_message\time_last_message_between_users',
     ),
 
+    // Caches font awesome icons.
+    'fontawesomeiconmapping' => array(
+        'mode' => cache_store::MODE_APPLICATION,
+        'simplekeys' => true,
+        'simpledata' => true,
+        'staticacceleration' => true,
+        'staticaccelerationsize' => 1
+    ),
+
     // Caches processed CSS.
     'postprocessedcss' => array(
         'mode' => cache_store::MODE_APPLICATION,
