@@ -30,6 +30,7 @@
  * this.filemanager, contains reference to filemanager Node
  * this.selectnode, contains referenct to select-file Node
  * this.selectui, M.core.dialogue to select the file
+ * this.viewmode, contains current view mode - icons, tree or details
  *
  * FileManager options:
  * =====
@@ -70,6 +71,7 @@ M.form_filemanager.init = function(Y, options) {
             this.maxfiles = options.maxfiles;
             this.maxbytes = options.maxbytes;
             this.areamaxbytes = options.areamaxbytes;
+            this.userprefs = options.userprefs;
             this.emptycallback = null; // Used by drag and drop upload
 
             this.filepicker_options = options.filepicker?options.filepicker:{};
@@ -103,7 +105,7 @@ M.form_filemanager.init = function(Y, options) {
                 this.pathbar.removeChild(this.pathnode);
             }
             // initialize 'select file' panel
-            this.selectnode = Y.Node.createWithFilesSkin(M.form_filemanager.templates.fileselectlayout);
+            this.selectnode = Y.Node.create(M.form_filemanager.templates.fileselectlayout);
             this.selectnode.setAttribute('aria-live', 'assertive');
             this.selectnode.setAttribute('role', 'dialog');
             this.selectnode.generateID();
@@ -111,7 +113,7 @@ M.form_filemanager.init = function(Y, options) {
             var labelid = 'fm-dialog-label_'+ this.selectnode.get('id');
             this.selectui = new M.core.dialogue({
                 draggable    : true,
-                headerContent: '<span id="' + labelid +'">' + M.str.moodle.edit + '</span>',
+                headerContent: '<h3 id="' + labelid +'">' + M.util.get_string('edit', 'moodle') + '</h3>',
                 bodyContent  : this.selectnode,
                 centered     : true,
                 width        : '480px',
@@ -126,9 +128,13 @@ M.form_filemanager.init = function(Y, options) {
             // set event handler for lazy loading of thumbnails
             this.filemanager.one('.fp-content').on(['scroll','resize'], this.content_scrolled, this);
             // display files
-            this.viewmode = 1; // TODO take from cookies?
-            this.filemanager.all('.fp-vb-icons,.fp-vb-tree,.fp-vb-details').removeClass('checked')
-            this.filemanager.all('.fp-vb-icons').addClass('checked')
+            this.viewmode = this.get_preference("recentviewmode");
+            if (this.viewmode != 2 && this.viewmode != 3) {
+                this.viewmode = 1;
+            }
+            var viewmodeselectors = {'1': '.fp-vb-icons', '2': '.fp-vb-tree', '3': '.fp-vb-details'};
+            this.filemanager.all('.fp-vb-icons,.fp-vb-tree,.fp-vb-details').removeClass('checked');
+            this.filemanager.all(viewmodeselectors[this.viewmode]).addClass('checked');
             this.refresh(this.currentpath); // MDL-31113 get latest list from server
         },
 
@@ -163,8 +169,8 @@ M.form_filemanager.init = function(Y, options) {
                         try {
                             data = Y.JSON.parse(o.responseText);
                         } catch(e) {
-                            scope.print_msg(M.str.repository.invalidjson, 'error');
-                            Y.error(M.str.repository.invalidjson+":\n"+o.responseText);
+                            scope.print_msg(M.util.get_string('invalidjson', 'repository'), 'error');
+                            Y.error(M.util.get_string('invalidjson', 'repository')+":\n"+o.responseText);
                             return;
                         }
                         if (data && data.tree && scope.set_current_tree) {
@@ -233,13 +239,13 @@ M.form_filemanager.init = function(Y, options) {
         },
         /** displays message in a popup */
         print_msg: function(msg, type) {
-            var header = M.str.moodle.error;
+            var header = M.util.get_string('error', 'moodle');
             if (type != 'error') {
                 type = 'info'; // one of only two types excepted
-                header = M.str.moodle.info;
+                header = M.util.get_string('info', 'moodle');
             }
             if (!this.msg_dlg) {
-                this.msg_dlg_node = Y.Node.createWithFilesSkin(M.form_filemanager.templates.message);
+                this.msg_dlg_node = Y.Node.create(M.form_filemanager.templates.message);
                 var nodeid = this.msg_dlg_node.generateID();
 
                 this.msg_dlg = new M.core.dialogue({
@@ -319,7 +325,7 @@ M.form_filemanager.init = function(Y, options) {
                         return valid;
                     };
                     if (!this.mkdir_dialog) {
-                        var node = Y.Node.createWithFilesSkin(M.form_filemanager.templates.mkdir);
+                        var node = Y.Node.create(M.form_filemanager.templates.mkdir);
                         this.mkdir_dialog = new M.core.dialogue({
                             draggable    : true,
                             bodyContent  : node,
@@ -346,7 +352,7 @@ M.form_filemanager.init = function(Y, options) {
                     this.mkdir_dialog.show();
 
                     // Default folder name:
-                    var foldername = M.str.repository.newfolder;
+                    var foldername = M.util.get_string('newfolder', 'repository');
                     while (this.has_folder(foldername)) {
                         foldername = increment_filename(foldername, true);
                     }
@@ -366,11 +372,21 @@ M.form_filemanager.init = function(Y, options) {
                     return;
                 }
                 var scope = this;
+
+                var image_downloading = this.filemanager.one('.fp-img-downloading');
+                if (image_downloading.getStyle('display') == 'inline') {
+                    return;
+                }
+                image_downloading.setStyle('display', 'inline');
+
                 // perform downloaddir ajax request
                 this.request({
                     action: 'downloaddir',
                     scope: scope,
                     callback: function(id, obj, args) {
+                        var image_downloading = scope.filemanager.one('.fp-img-downloading');
+                        image_downloading.setStyle('display', 'none');
+
                         if (obj) {
                             scope.refresh(obj.filepath);
                             node = Y.Node.create('<iframe></iframe>').setStyles({
@@ -381,7 +397,7 @@ M.form_filemanager.init = function(Y, options) {
                             node.set('src', obj.fileurl);
                             Y.one('body').appendChild(node);
                         } else {
-                            scope.print_msg(M.str.repository.draftareanofiles, 'error');
+                            scope.print_msg(M.util.get_string('draftareanofiles', 'repository'), 'error');
                         }
                     }
                 });
@@ -404,6 +420,7 @@ M.form_filemanager.init = function(Y, options) {
                         this.render();
                         this.filemanager.one('.fp-content').setAttribute('tabIndex', '0');
                         this.filemanager.one('.fp-content').focus();
+                        this.set_preference('recentviewmode', this.viewmode);
                     }
                 }, this);
         },
@@ -675,12 +692,12 @@ M.form_filemanager.init = function(Y, options) {
             var dialog_options = {callback:this.update_file, callbackargs:[true], scope:this};
             if (fileinfo.type == 'folder') {
                 if (!newfilename) {
-                    this.print_msg(M.str.repository.entername, 'error');
+                    this.print_msg(M.util.get_string('entername', 'repository'), 'error');
                     return;
                 }
                 if (filenamechanged || filepathchanged) {
                     if (!confirmed) {
-                        dialog_options.message = M.str.repository.confirmrenamefolder;
+                        dialog_options.message = M.util.get_string('confirmrenamefolder', 'repository');
                         this.show_confirm_dialog(dialog_options);
                         return;
                     }
@@ -689,7 +706,7 @@ M.form_filemanager.init = function(Y, options) {
                 }
             } else {
                 if (!newfilename) {
-                    this.print_msg(M.str.repository.enternewname, 'error');
+                    this.print_msg(M.util.get_string('enternewname', 'repository'), 'error');
                     return;
                 }
                 if ((filenamechanged || filepathchanged) && !confirmed && fileinfo.refcount) {
@@ -735,7 +752,7 @@ M.form_filemanager.init = function(Y, options) {
         show_confirm_dialog: function(dialog_options) {
             // instead of M.util.show_confirm_dialog(e, dialog_options);
             if (!this.confirm_dlg) {
-                this.confirm_dlg_node = Y.Node.createWithFilesSkin(M.form_filemanager.templates.confirmdialog);
+                this.confirm_dlg_node = Y.Node.create(M.form_filemanager.templates.confirmdialog);
                 var node = this.confirm_dlg_node;
                 node.generateID();
                 this.confirm_dlg = new M.core.dialogue({
@@ -808,13 +825,13 @@ M.form_filemanager.init = function(Y, options) {
                 params.filepath = fileinfo.filepath;
                 if (fileinfo.type == 'folder') {
                     params.filename = '.';
-                    dialog_options.message = M.str.repository.confirmdeletefolder;
+                    dialog_options.message = M.util.get_string('confirmdeletefolder', 'repository');
                 } else {
                     params.filename = fileinfo.fullname;
                     if (fileinfo.refcount) {
                         dialog_options.message = M.util.get_string('confirmdeletefilewithhref', 'repository', fileinfo.refcount);
                     } else {
-                        dialog_options.message = M.str.repository.confirmdeletefile;
+                        dialog_options.message = M.util.get_string('confirmdeletefile', 'repository');
                     }
                 }
                 dialog_options.callbackargs = [params];
@@ -950,8 +967,12 @@ M.form_filemanager.init = function(Y, options) {
             for (var i in attrs) {
                 if (selectnode.one('.fp-'+attrs[i])) {
                     var value = (node[attrs[i]+'_f']) ? node[attrs[i]+'_f'] : (node[attrs[i]] ? node[attrs[i]] : '');
+                    // Escape if the attribute being evaluated is not for the list of reference files.
+                    if (attrs[i] !== 'reflist') {
+                        value = Y.Escape.html(value);
+                    }
                     selectnode.one('.fp-'+attrs[i]).addClassIf('fp-unknown', ''+value == '')
-                        .one('.fp-value').setContent(Y.Escape.html(value));
+                        .one('.fp-value').setContent(value);
                 }
             }
             // display thumbnail
@@ -978,7 +999,7 @@ M.form_filemanager.init = function(Y, options) {
                                 node.original = obj.original;
                                 selectnode.one('.fp-original .fp-value').setContent(Y.Escape.html(node.original));
                             } else {
-                                selectnode.one('.fp-original .fp-value').setContent(M.str.repository.unknownsource);
+                                selectnode.one('.fp-original .fp-value').setContent(M.util.get_string('unknownsource', 'repository'));
                             }
                         }
                     }
@@ -1019,7 +1040,7 @@ M.form_filemanager.init = function(Y, options) {
             if (nodename.length > namelength) {
                 nodename = nodename.substring(0, namelength) + '...';
             }
-            Y.one('#fm-dialog-label_'+selectnode.get('id')).setContent(Y.Escape.html(M.str.moodle.edit+' '+nodename));
+            Y.one('#fm-dialog-label_'+selectnode.get('id')).setContent(Y.Escape.html(M.util.get_string('edit', 'moodle')+' '+nodename));
             // show panel
             this.selectui.show();
             Y.one('#'+selectnode.get('id')).focus();
@@ -1037,7 +1058,20 @@ M.form_filemanager.init = function(Y, options) {
                 }
             }
             return false;
-        }
+        },
+        get_preference: function(name) {
+            if (this.userprefs[name]) {
+                return this.userprefs[name];
+            } else {
+                return false;
+            }
+        },
+        set_preference: function(name, value) {
+            if (this.userprefs[name] != value) {
+                M.util.set_user_preference('filemanager_' + name, value);
+                this.userprefs[name] = value;
+            }
+        },
     });
 
     // finally init everything needed
