@@ -1,3 +1,4 @@
+/* eslint-disable no-empty-function */
 /**
  * The core drag and drop module for Moodle which extends the YUI drag and
  * drop functionality with additional features.
@@ -69,7 +70,7 @@ Y.extend(DRAGDROP, Y.Base, {
      * @type Object
      * @default null
      */
-    samenodelabel : null,
+    samenodelabel: null,
 
     /**
      * The label to use with keyboard drag/drop to describe items of the parent Node.
@@ -78,7 +79,7 @@ Y.extend(DRAGDROP, Y.Base, {
      * @type Object
      * @default null
      */
-    parentnodelabel : null,
+    parentnodelabel: null,
 
     /**
      * The groups for this instance.
@@ -98,6 +99,15 @@ Y.extend(DRAGDROP, Y.Base, {
      */
     lastdroptarget: null,
 
+    /**
+     * Listeners.
+     *
+     * @property listeners
+     * @type Array
+     * @default null
+     */
+    listeners: null,
+
     /**
      * The initializer which sets up the move action.
      *
@@ -105,31 +115,43 @@ Y.extend(DRAGDROP, Y.Base, {
      * @protected
      */
     initializer: function() {
+        this.listeners = [];
+
         // Listen for all drag:start events.
-        Y.DD.DDM.on('drag:start', this.global_drag_start, this);
+        this.listeners.push(Y.DD.DDM.on('drag:start', this.global_drag_start, this));
 
         // Listen for all drag:end events.
-        Y.DD.DDM.on('drag:end', this.global_drag_end, this);
+        this.listeners.push(Y.DD.DDM.on('drag:end', this.global_drag_end, this));
 
         // Listen for all drag:drag events.
-        Y.DD.DDM.on('drag:drag', this.global_drag_drag, this);
+        this.listeners.push(Y.DD.DDM.on('drag:drag', this.global_drag_drag, this));
 
         // Listen for all drop:over events.
-        Y.DD.DDM.on('drop:over', this.global_drop_over, this);
+        this.listeners.push(Y.DD.DDM.on('drop:over', this.global_drop_over, this));
 
         // Listen for all drop:hit events.
-        Y.DD.DDM.on('drop:hit', this.global_drop_hit, this);
+        this.listeners.push(Y.DD.DDM.on('drop:hit', this.global_drop_hit, this));
 
         // Listen for all drop:miss events.
-        Y.DD.DDM.on('drag:dropmiss', this.global_drag_dropmiss, this);
+        this.listeners.push(Y.DD.DDM.on('drag:dropmiss', this.global_drag_dropmiss, this));
 
         // Add keybaord listeners for accessible drag/drop
-        Y.one(Y.config.doc.body).delegate('key', this.global_keydown,
-                'down:32, enter, esc', '.' + MOVEICON.cssclass, this);
+        this.listeners.push(Y.one(Y.config.doc.body).delegate('key', this.global_keydown,
+                'down:32, enter, esc', '.' + MOVEICON.cssclass, this));
 
         // Make the accessible drag/drop respond to a single click.
-        Y.one(Y.config.doc.body).delegate('click', this.global_keydown,
-                '.' + MOVEICON.cssclass , this);
+        this.listeners.push(Y.one(Y.config.doc.body).delegate('click', this.global_keydown,
+                '.' + MOVEICON.cssclass, this));
+    },
+
+    /**
+     * The destructor to shut down the instance of the dragdrop system.
+     *
+     * @method destructor
+     * @protected
+     */
+    destructor: function() {
+        new Y.EventHandle(this.listeners).detach();
     },
 
     /**
@@ -139,25 +161,10 @@ Y.extend(DRAGDROP, Y.Base, {
      * @param {String} title The title on the drag handle
      * @param {String} classname The name of the class to add to the node
      * wrapping the drag icon
-     * @param {String} [iconclass] The class to add to the icon
-     * @param {Boolean} [large=false] whether to use the larger version of
-     * the drag icon
+     * @param {String} iconclass Additional class to add to the icon.
      * @return Node The built drag handle.
      */
-    get_drag_handle: function(title, classname, iconclass, large) {
-        var iconname = MOVEICON.pix;
-        if (large) {
-            iconname = MOVEICON.largepix;
-        }
-        var dragicon = Y.Node.create('<img />')
-            .setStyle('cursor', 'move')
-            .setAttrs({
-                'src': M.util.image_url(iconname, MOVEICON.component),
-                'alt': title
-            });
-        if (iconclass) {
-            dragicon.addClass(iconclass);
-        }
+    get_drag_handle: function(title, classname, iconclass) {
 
         var dragelement = Y.Node.create('<span></span>')
             .addClass(classname)
@@ -165,24 +172,34 @@ Y.extend(DRAGDROP, Y.Base, {
             .setAttribute('tabIndex', 0)
             .setAttribute('data-draggroups', this.groups)
             .setAttribute('role', 'button');
-        dragelement.appendChild(dragicon);
         dragelement.addClass(MOVEICON.cssclass);
 
+        window.require(['core/templates'], function(Templates) {
+            Templates.renderPix('i/move_2d', 'core').then(function(html) {
+                var dragicon = Y.Node.create(html);
+                dragicon.setStyle('cursor', 'move');
+                if (typeof iconclass != 'undefined') {
+                    dragicon.addClass(iconclass);
+                }
+                dragelement.appendChild(dragicon);
+            });
+        });
+
         return dragelement;
     },
 
     lock_drag_handle: function(drag, classname) {
-        drag.removeHandle('.'+classname);
+        drag.removeHandle('.' + classname);
     },
 
     unlock_drag_handle: function(drag, classname) {
-        drag.addHandle('.'+classname);
+        drag.addHandle('.' + classname);
         drag.get('activeHandle').focus();
     },
 
     ajax_failure: function(response) {
         var e = {
-            name: response.status+' '+response.statusText,
+            name: response.status + ' ' + response.statusText,
             message: response.responseText
         };
         return new M.core.exception(e);
@@ -207,6 +224,8 @@ Y.extend(DRAGDROP, Y.Base, {
         if (!this.in_group(drag)) {
             return;
         }
+        // Store the nodes current style, so we can restore it later.
+        this.originalstyle = drag.get('node').getAttribute('style');
         // Set some general styles here
         drag.get('node').setStyle('opacity', '.25');
         drag.get('dragNode').setStyles({
@@ -224,11 +243,8 @@ Y.extend(DRAGDROP, Y.Base, {
         if (!this.in_group(drag)) {
             return;
         }
-        //Put our general styles back
-        drag.get('node').setStyles({
-            visibility: '',
-            opacity: '1'
-        });
+        // Put our general styles back
+        drag.get('node').setAttribute('style', this.originalstyle);
         this.drag_end(e);
     },
 
@@ -337,11 +353,27 @@ Y.extend(DRAGDROP, Y.Base, {
      * @return {string} The text of the first text-like child node of n.
      */
     find_element_text: function(n) {
-        // The valid node types to get text from.
-        var nodes = n.all('h2, h3, h4, h5, span, p, div.no-overflow, div.dimmed_text');
         var text = '';
 
-        nodes.each(function () {
+        // Try to resolve using aria-label first.
+        text = n.get('aria-label') || '';
+        if (text.length > 0) {
+            return text;
+        }
+
+        // Now try to resolve using aria-labelledby.
+        var labelledByNode = n.get('aria-labelledby');
+        if (labelledByNode) {
+            var labelNode = Y.one('#' + labelledByNode);
+            if (labelNode && labelNode.get('text').length > 0) {
+                return labelNode.get('text');
+            }
+        }
+
+        // The valid node types to get text from.
+        var nodes = n.all('h2, h3, h4, h5, span:not(.actions):not(.menu-action-text), p, div.no-overflow, div.dimmed_text');
+
+        nodes.each(function() {
             if (text === '') {
                 if (Y.Lang.trim(this.get('text')) !== '') {
                     text = this.get('text');
@@ -375,16 +407,18 @@ Y.extend(DRAGDROP, Y.Base, {
         // Build the list of drop targets.
         var droplist = Y.Node.create('<ul></ul>');
         droplist.addClass('dragdrop-keyboard-drag');
-        var listitem,
-            listlink,
-            listitemtext;
+        var listitem, listlink, listitemtext;
 
         // Search for possible drop targets.
         var droptargets = Y.all('.' + this.samenodeclass + ', .' + this.parentnodeclass);
 
-        droptargets.each(function (node) {
-            var validdrop = false, labelroot = node;
-            if (node.drop && node.drop.inGroup(this.groups) && node.drop.get('node') !== dragcontainer) {
+        droptargets.each(function(node) {
+            var validdrop = false;
+            var labelroot = node;
+            var className = node.getAttribute("class").split(' ').join(', .');
+
+            if (node.drop && node.drop.inGroup(this.groups) && node.drop.get('node') !== dragcontainer &&
+                    node.next(className) !== dragcontainer) {
                 // This is a drag and drop target with the same class as the grabbed node.
                 validdrop = true;
             } else {
@@ -392,12 +426,13 @@ Y.extend(DRAGDROP, Y.Base, {
                 var i, j;
                 for (i = 0; i < elementgroups.length; i++) {
                     for (j = 0; j < this.groups.length; j++) {
-                        if (elementgroups[i] === this.groups[j]) {
-                            // This is a parent node of the grabbed node (used for dropping in empty sections).
-                            validdrop = true;
-                            // This node will have no text - so we get the first valid text from the parent.
-                            labelroot = node.get('parentNode');
-                            break;
+                        if (elementgroups[i] === this.groups[j] && !(node == dragcontainer ||
+                            node.next(className) === dragcontainer || node.get('children').item(0) == dragcontainer)) {
+                                // This is a parent node of the grabbed node (used for dropping in empty sections).
+                                validdrop = true;
+                                // This node will have no text - so we get the first valid text from the parent.
+                                labelroot = node.get('parentNode');
+                                break;
                         }
                     }
                     if (validdrop) {
@@ -581,7 +616,7 @@ Y.extend(DRAGDROP, Y.Base, {
             return;
         }
 
-        if (e.keyCode === 27 ) {
+        if (e.keyCode === 27) {
             // Escape to cancel from anywhere.
             this.global_cancel_keyboard_drag();
             e.preventDefault();
@@ -600,7 +635,8 @@ Y.extend(DRAGDROP, Y.Base, {
 
         // Check the drag groups to see if we are the handler for this node.
         draggroups = draghandle.getAttribute('data-draggroups').split(' ');
-        var i, j, validgroup = false;
+        var i, j;
+        var validgroup = false;
 
         for (i = 0; i < draggroups.length; i++) {
             for (j = 0; j < this.groups.length; j++) {
