@@ -23,6 +23,8 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+defined('MOODLE_INTERNAL') || die();
+
 require_once("$CFG->libdir/externallib.php");
 
 /**
@@ -64,7 +66,7 @@ class core_user_external extends external_api {
                             'email' =>
                                 new external_value(core_user::get_property_type('email'), 'A valid and unique email address'),
                             'auth' =>
-                                new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, imap, etc', VALUE_DEFAULT,
+                                new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, etc', VALUE_DEFAULT,
                                     'manual', core_user::get_property_null('auth')),
                             'idnumber' =>
                                 new external_value(core_user::get_property_type('idnumber'), 'An arbitrary ID code number perhaps from the institution',
@@ -229,9 +231,11 @@ class core_user_external extends external_api {
 
             // Preferences.
             if (!empty($user['preferences'])) {
+                $userpref = (object)$user;
                 foreach ($user['preferences'] as $preference) {
-                    set_user_preference($preference['type'], $preference['value'], $user['id']);
+                    $userpref->{'preference_'.$preference['type']} = $preference['value'];
                 }
+                useredit_update_user_preference($userpref);
             }
 
             $userids[] = array('id' => $user['id'], 'username' => $user['username']);
@@ -359,6 +363,8 @@ class core_user_external extends external_api {
         global $USER, $CFG;
 
         require_once($CFG->dirroot . '/user/lib.php');
+        require_once($CFG->dirroot . '/user/editlib.php');
+        require_once($CFG->dirroot . '/message/lib.php');
 
         if (empty($userid)) {
             $userid = $USER->id;
@@ -373,38 +379,29 @@ class core_user_external extends external_api {
         );
         self::validate_parameters(self::update_user_preferences_parameters(), $params);
 
-        if ($userid == $USER->id) {
-            require_capability('moodle/user:editownmessageprofile', $systemcontext);
-        } else {
-            $personalcontext = context_user::instance($userid);
-            require_capability('moodle/user:editmessageprofile', $personalcontext);
-            // No editing of guest user account.
-            if (isguestuser($userid)) {
-                print_error('guestnoeditmessageother', 'message');
-            }
-            // No editing of admins by non-admins.
-            if (is_siteadmin($userid) and !is_siteadmin($USER)) {
-                print_error('useradmineditadmin');
-            }
-        }
-
         // Preferences.
         if (!empty($preferences)) {
+            $userpref = ['id' => $userid];
             foreach ($preferences as $preference) {
-                set_user_preference($preference['type'], $preference['value'], $userid);
+                $userpref['preference_' . $preference['type']] = $preference['value'];
             }
+            useredit_update_user_preference($userpref);
         }
 
         // Check if they want to update the email.
         if ($emailstop !== null) {
-            $user = new stdClass();
-            $user->id = $userid;
-            $user->emailstop = $emailstop;
-            user_update_user($user);
+            $otheruser = ($userid == $USER->id) ? $USER : core_user::get_user($userid, '*', MUST_EXIST);
+            core_user::require_active_user($otheruser);
+            if (core_message_can_edit_message_profile($otheruser) && $otheruser->emailstop != $emailstop) {
+                $user = new stdClass();
+                $user->id = $userid;
+                $user->emailstop = $emailstop;
+                user_update_user($user);
 
-            // Update the $USER if we should.
-            if ($userid == $USER->id) {
-                $USER->emailstop = $emailstop;
+                // Update the $USER if we should.
+                if ($userid == $USER->id) {
+                    $USER->emailstop = $emailstop;
+                }
             }
         }
 
@@ -450,7 +447,7 @@ class core_user_external extends external_api {
                                 new external_value(core_user::get_property_type('email'), 'A valid and unique email address', VALUE_OPTIONAL, '',
                                     NULL_NOT_ALLOWED),
                             'auth' =>
-                                new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, imap, etc', VALUE_OPTIONAL, '',
+                                new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, etc', VALUE_OPTIONAL, '',
                                     NULL_NOT_ALLOWED),
                             'suspended' =>
                                 new external_value(core_user::get_property_type('suspended'), 'Suspend user account, either false to enable user login or true to disable it', VALUE_OPTIONAL),
@@ -521,6 +518,7 @@ class core_user_external extends external_api {
         global $CFG, $DB, $USER;
         require_once($CFG->dirroot."/user/lib.php");
         require_once($CFG->dirroot."/user/profile/lib.php"); // Required for customfields related function.
+        require_once($CFG->dirroot.'/user/editlib.php');
 
         // Ensure the current user is allowed to run this function.
         $context = context_system::instance();
@@ -549,6 +547,16 @@ class core_user_external extends external_api {
             if ($existinguser->deleted or is_mnet_remote_user($existinguser) or isguestuser($existinguser->id)) {
                 continue;
             }
+            // Check duplicated emails.
+            if (isset($user['email']) && $user['email'] !== $existinguser->email) {
+                if (!validate_email($user['email'])) {
+                    continue;
+                } else if (empty($CFG->allowaccountssameemail) &&
+                        $DB->record_exists('user', array('email' => $user['email'], 'mnethostid' => $CFG->mnet_localhost_id))) {
+                    continue;
+                }
+            }
+
             user_update_user($user, true, false);
 
             // Update user picture if it was specified for this user.
@@ -582,9 +590,11 @@ class core_user_external extends external_api {
 
             // Preferences.
             if (!empty($user['preferences'])) {
+                $userpref = clone($existinguser);
                 foreach ($user['preferences'] as $preference) {
-                    set_user_preference($preference['type'], $preference['value'], $user['id']);
+                    $userpref->{'preference_'.$preference['type']} = $preference['value'];
                 }
+                useredit_update_user_preference($userpref);
             }
             if (isset($user['suspended']) and $user['suspended']) {
                 \core\session\manager::kill_user_sessions($user['id']);
@@ -1032,7 +1042,7 @@ class core_user_external extends external_api {
             'interests'   => new external_value(PARAM_TEXT, 'user interests (separated by commas)', VALUE_OPTIONAL),
             'firstaccess' => new external_value(core_user::get_property_type('firstaccess'), 'first access to the site (0 if never)', VALUE_OPTIONAL),
             'lastaccess'  => new external_value(core_user::get_property_type('lastaccess'), 'last access to the site (0 if never)', VALUE_OPTIONAL),
-            'auth'        => new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, imap, etc', VALUE_OPTIONAL),
+            'auth'        => new external_value(core_user::get_property_type('auth'), 'Auth plugins include manual, ldap, etc', VALUE_OPTIONAL),
             'suspended'   => new external_value(core_user::get_property_type('suspended'), 'Suspend user account, either false to enable user login or true to disable it', VALUE_OPTIONAL),
             'confirmed'   => new external_value(core_user::get_property_type('confirmed'), 'Active user: 1 if confirmed, 0 otherwise', VALUE_OPTIONAL),
             'lang'        => new external_value(core_user::get_property_type('lang'), 'Language code such as "en", must exist on server', VALUE_OPTIONAL),
@@ -1332,6 +1342,7 @@ class core_user_external extends external_api {
     public static function view_user_list($courseid) {
         global $CFG;
         require_once($CFG->dirroot . "/user/lib.php");
+        require_once($CFG->dirroot . '/course/lib.php');
 
         $params = self::validate_parameters(self::view_user_list_parameters(),
                                             array(
@@ -1353,11 +1364,7 @@ class core_user_external extends external_api {
         }
         self::validate_context($context);
 
-        if ($course->id == SITEID) {
-            require_capability('moodle/site:viewparticipants', $context);
-        } else {
-            require_capability('moodle/course:viewparticipants', $context);
-        }
+        course_require_view_participants($context);
 
         user_list_view($course, $context);
 
@@ -1710,7 +1717,6 @@ class core_user_external extends external_api {
 
         $context = context_system::instance();
         self::validate_context($context);
-        require_capability('moodle/site:config', $context);
 
         $userscache = array();
         foreach ($params['preferences'] as $pref) {
@@ -1734,11 +1740,21 @@ class core_user_external extends external_api {
             }
 
             try {
-                set_user_preference($pref['name'], $pref['value'], $user);
-                $saved[] = array(
-                    'name' => $pref['name'],
-                    'userid' => $user->id,
-                );
+                if (core_user::can_edit_preference($pref['name'], $user)) {
+                    $value = core_user::clean_preference($pref['value'], $pref['name']);
+                    set_user_preference($pref['name'], $value, $user->id);
+                    $saved[] = array(
+                        'name' => $pref['name'],
+                        'userid' => $user->id,
+                    );
+                } else {
+                    $warnings[] = array(
+                        'item' => 'user',
+                        'itemid' => $user->id,
+                        'warningcode' => 'nopermission',
+                        'message' => 'You are not allowed to change the preference '.s($pref['name']).' for user '.$user->id
+                    );
+                }
             } catch (Exception $e) {
                 $warnings[] = array(
                     'item' => 'user',
@@ -1853,4 +1869,76 @@ class core_user_external extends external_api {
             )
         );
     }
+
+    /**
+     * Returns description of method parameters.
+     *
+     * @return external_function_parameters
+     * @since Moodle 3.4
+     */
+    public static function get_private_files_info_parameters() {
+        return new external_function_parameters(
+            array(
+                'userid' => new external_value(PARAM_INT, 'Id of the user, default to current user.', VALUE_DEFAULT, 0)
+            )
+        );
+    }
+
+    /**
+     * Returns general information about files in the user private files area.
+     *
+     * @param int $userid Id of the user, default to current user.
+     * @return array of warnings and file area information
+     * @since Moodle 3.4
+     * @throws moodle_exception
+     */
+    public static function get_private_files_info($userid = 0) {
+        global $CFG, $USER;
+        require_once($CFG->libdir . '/filelib.php');
+
+        $params = self::validate_parameters(self::get_private_files_info_parameters(), array('userid' => $userid));
+        $warnings = array();
+
+        $context = context_system::instance();
+        self::validate_context($context);
+
+        if (empty($params['userid']) || $params['userid'] == $USER->id) {
+            $usercontext = context_user::instance($USER->id);
+            require_capability('moodle/user:manageownfiles', $usercontext);
+        } else {
+            $user = core_user::get_user($params['userid'], '*', MUST_EXIST);
+            core_user::require_active_user($user);
+            // Only admins can retrieve other users information.
+            require_capability('moodle/site:config', $context);
+            $usercontext = context_user::instance($user->id);
+        }
+
+        $fileareainfo = file_get_file_area_info($usercontext->id, 'user', 'private');
+
+        $result = array();
+        $result['filecount'] = $fileareainfo['filecount'];
+        $result['foldercount'] = $fileareainfo['foldercount'];
+        $result['filesize'] = $fileareainfo['filesize'];
+        $result['filesizewithoutreferences'] = $fileareainfo['filesize_without_references'];
+        $result['warnings'] = $warnings;
+        return $result;
+    }
+
+    /**
+     * Returns description of method result value.
+     *
+     * @return external_description
+     * @since Moodle 3.4
+     */
+    public static function get_private_files_info_returns() {
+        return new external_single_structure(
+            array(
+                'filecount' => new external_value(PARAM_INT, 'Number of files in the area.'),
+                'foldercount' => new external_value(PARAM_INT, 'Number of folders in the area.'),
+                'filesize' => new external_value(PARAM_INT, 'Total size of the files in the area.'),
+                'filesizewithoutreferences' => new external_value(PARAM_INT, 'Total size of the area excluding file references'),
+                'warnings' => new external_warnings()
+            )
+        );
+    }
 }
