@@ -494,11 +494,19 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             'email' => 'usertest1@example.com',
             'description' => 'This is a description for user 1',
             'city' => 'Perth',
-            'country' => 'AU'
+            'country' => 'AU',
+            'preferences' => [[
+                    'type' => 'htmleditor',
+                    'value' => 'atto'
+                ], [
+                    'type' => 'invalidpreference',
+                    'value' => 'abcd'
+                ]]
             );
 
         $context = context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:create', $context->id);
+        $this->assignUserCapability('moodle/user:editprofile', $context->id, $roleid);
 
         // Call the external function.
         $createdusers = core_user_external::create_users(array($user1));
@@ -519,6 +527,8 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             $this->assertEquals($dbuser->description, $user1['description']);
             $this->assertEquals($dbuser->city, $user1['city']);
             $this->assertEquals($dbuser->country, $user1['country']);
+            $this->assertEquals('atto', get_user_preferences('htmleditor', null, $dbuser));
+            $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
         }
 
         // Call without required capability
@@ -599,11 +609,19 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             'description' => 'This is a description for user 1',
             'city' => 'Perth',
             'userpicture' => $draftid,
-            'country' => 'AU'
+            'country' => 'AU',
+            'preferences' => [[
+                    'type' => 'htmleditor',
+                    'value' => 'atto'
+                ], [
+                    'type' => 'invialidpreference',
+                    'value' => 'abcd'
+                ]]
             );
 
         $context = context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:update', $context->id);
+        $this->assignUserCapability('moodle/user:editprofile', $context->id, $roleid);
 
         // Check we can't update deleted users, guest users, site admin.
         $user2 = $user3 = $user4 = $user1;
@@ -636,6 +654,8 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals($dbuser->city, $user1['city']);
         $this->assertEquals($dbuser->country, $user1['country']);
         $this->assertNotEquals(0, $dbuser->picture, 'Picture must be set to the new icon itemid for this user');
+        $this->assertEquals('atto', get_user_preferences('htmleditor', null, $dbuser));
+        $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
 
         // Confirm no picture change when parameter is not supplied.
         unset($user1['userpicture']);
@@ -656,6 +676,31 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         core_user_external::update_users(array($user1));
     }
 
+    /**
+     * Test update_users using duplicated email.
+     */
+    public function test_update_users_duplicated_email() {
+        global $DB, $CFG;
+
+        $this->resetAfterTest(true);
+        $this->setAdminUser();
+
+        $user1 = self::getDataGenerator()->create_user();
+        $user2 = self::getDataGenerator()->create_user();
+        $user2toupdate = array(
+            'id' => $user2->id,
+            'email' => $user1->email,
+        );
+        // E-mail duplicated not allowed.
+        $CFG->allowaccountssameemail = 0;
+        core_user_external::update_users(array($user2toupdate));
+        $this->assertNotEquals($user1->email, $DB->get_field('user', 'email', array('id' => $user2->id)));
+        // E-mail duplicated allowed.
+        $CFG->allowaccountssameemail = 1;
+        core_user_external::update_users(array($user2toupdate));
+        $this->assertEquals($user1->email, $DB->get_field('user', 'email', array('id' => $user2->id)));
+    }
+
     /**
      * Test add_user_private_files
      */
@@ -868,7 +913,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $anotheruser = self::getDataGenerator()->create_user();
         $this->setUser($anotheruser);
 
-        $this->setExpectedException('required_capability_exception');
+        $this->expectException('required_capability_exception');
         $result = core_user_external::get_user_preferences('', $user->id);
     }
 
@@ -952,13 +997,13 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->setAdminUser();
         $preferences = array(
             array(
-                'name' => 'some_random_pref',
-                'value' => 'abc',
+                'name' => 'htmleditor',
+                'value' => 'atto',
                 'userid' => $user1->id,
             ),
             array(
-                'name' => 'some_random_pref',
-                'value' => 'def',
+                'name' => 'htmleditor',
+                'value' => 'tinymce',
                 'userid' => $user2->id,
             )
         );
@@ -969,10 +1014,39 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->assertCount(2, $result['saved']);
 
         // Get preference from DB to avoid cache.
-        $this->assertEquals('abc', $DB->get_field('user_preferences', 'value',
-            array('userid' => $user1->id, 'name' => 'some_random_pref')));
-        $this->assertEquals('def', $DB->get_field('user_preferences', 'value',
-            array('userid' => $user2->id, 'name' => 'some_random_pref')));
+        $this->assertEquals('atto', $DB->get_field('user_preferences', 'value',
+            array('userid' => $user1->id, 'name' => 'htmleditor')));
+        $this->assertEquals('tinymce', $DB->get_field('user_preferences', 'value',
+            array('userid' => $user2->id, 'name' => 'htmleditor')));
+    }
+
+    /**
+     * Test set_user_preferences
+     */
+    public function test_set_user_preferences_save_invalid_pref() {
+        global $DB;
+        $this->resetAfterTest(true);
+
+        $user1 = self::getDataGenerator()->create_user();
+
+        // Save users preferences.
+        $this->setAdminUser();
+        $preferences = array(
+            array(
+                'name' => 'some_random_pref',
+                'value' => 'abc',
+                'userid' => $user1->id,
+            ),
+        );
+
+        $result = core_user_external::set_user_preferences($preferences);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $this->assertCount(1, $result['warnings']);
+        $this->assertCount(0, $result['saved']);
+        $this->assertEquals('nopermission', $result['warnings'][0]['warningcode']);
+
+        // Nothing was written to DB.
+        $this->assertEmpty($DB->count_records('user_preferences', array('name' => 'some_random_pref')));
     }
 
     /**
@@ -1002,7 +1076,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
      * Test set_user_preferences using an invalid preference
      */
     public function test_set_user_preferences_invalid_preference() {
-        global $USER;
+        global $USER, $DB;
 
         $this->resetAfterTest(true);
         // Create a very long value.
@@ -1017,15 +1091,17 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         $result = core_user_external::set_user_preferences($preferences);
         $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
-        $this->assertCount(1, $result['warnings']);
-        $this->assertCount(0, $result['saved']);
-        $this->assertEquals('errorsavingpreference', $result['warnings'][0]['warningcode']);
+        $this->assertCount(0, $result['warnings']);
+        $this->assertCount(1, $result['saved']);
+        // Cleaned valud of the preference was saved.
+        $this->assertEquals(1, $DB->get_field('user_preferences', 'value',
+            array('userid' => $USER->id, 'name' => 'calendar_maxevents')));
     }
 
     /**
      * Test set_user_preferences for other user not being admin
      */
-    public function test_set_user_preferences_other_user_not_being_admin() {
+    public function test_set_user_preferences_capability() {
         $this->resetAfterTest(true);
 
         $user1 = self::getDataGenerator()->create_user();
@@ -1040,8 +1116,12 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             )
         );
 
-        $this->expectException('required_capability_exception');
         $result = core_user_external::set_user_preferences($preferences);
+
+        $this->assertCount(1, $result['warnings']);
+        $this->assertCount(0, $result['saved']);
+        $this->assertEquals('nopermission', $result['warnings'][0]['warningcode']);
+        $this->assertEquals($user2->id, $result['warnings'][0]['itemid']);
     }
 
     /**
@@ -1093,6 +1173,60 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         } catch (Exception $e) {
             $this->fail('Expecting \'usernotfullysetup\' moodle_exception to be thrown.');
         }
+    }
 
+    /**
+     * Test get_private_files_info
+     */
+    public function test_get_private_files_info() {
+
+        $this->resetAfterTest(true);
+        $user = self::getDataGenerator()->create_user();
+        $this->setUser($user);
+        $usercontext = context_user::instance($user->id);
+
+        $filerecord = array(
+            'contextid' => $usercontext->id,
+            'component' => 'user',
+            'filearea'  => 'private',
+            'itemid'    => 0,
+            'filepath'  => '/',
+            'filename'  => 'thefile',
+        );
+
+        $fs = get_file_storage();
+        $file = $fs->create_file_from_string($filerecord, 'abc');
+
+        // Get my private files information.
+        $result = core_user_external::get_private_files_info();
+        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $this->assertEquals(1, $result['filecount']);
+        $this->assertEquals($file->get_filesize(), $result['filesize']);
+        $this->assertEquals(0, $result['foldercount']);
+        $this->assertEquals($file->get_filesize(), $result['filesizewithoutreferences']);
+
+        // As admin, get user information.
+        $this->setAdminUser();
+        $result = core_user_external::get_private_files_info($user->id);
+        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $this->assertEquals(1, $result['filecount']);
+        $this->assertEquals($file->get_filesize(), $result['filesize']);
+        $this->assertEquals(0, $result['foldercount']);
+        $this->assertEquals($file->get_filesize(), $result['filesizewithoutreferences']);
+    }
+
+    /**
+     * Test get_private_files_info missing permissions.
+     */
+    public function test_get_private_files_info_missing_permissions() {
+
+        $this->resetAfterTest(true);
+        $user1 = self::getDataGenerator()->create_user();
+        $user2 = self::getDataGenerator()->create_user();
+        $this->setUser($user1);
+
+        $this->expectException('required_capability_exception');
+        // Try to retrieve other user private files info.
+        core_user_external::get_private_files_info($user2->id);
     }
 }
